@@ -61,8 +61,6 @@ static char sccsid[] = "@(#)ftpd.c	5.27.1.1	(Berkeley) 3/2/89";
 #include <netdb.h>
 #include <errno.h>
 #include <string.h>
-#define index	strchr
-#define rindex	strrchr
 #include <syslog.h>
 
 #include <sys/time.h>
@@ -167,10 +165,25 @@ char	*LastArgv = NULL;	/* end of argv */
 char	proctitle[BUFSIZ];	/* initial part of title */
 #endif /* SETPROCTITLE */
 
+/* SKEY or SNK interface definition: both use the same interface */
 #ifdef SKEY
+#define keyaccess skeyaccess
+#define key_challenge skey_challenge
+#define key_crypt skey_crypt
+#define KEY
+#endif /* SKEY */
+
+#ifdef SNK
+#define keyaccess snkaccess
+#define key_challenge snk_ftpprompt
+#define key_crypt snk_crypt
+#define KEY
+#endif /* SNK */
+
+#ifdef KEY
 int	pwok = 0;
-char	*skey_challenge();
-char	*skey_crypt();
+char	*key_challenge();
+char	*key_crypt();
 char	addr_string[20];	/* XXX */
 #endif
 
@@ -190,7 +203,7 @@ main(argc, argv, envp)
 		syslog(LOG_ERR, "getpeername (%s): %m",argv[0]);
 		exit(1);
 	}
-#ifdef SKEY
+#ifdef KEY
 	strcpy(addr_string, inet_ntoa(his_addr.sin_addr));
 #endif
 	addrlen = sizeof (ctrl_addr);
@@ -201,7 +214,7 @@ main(argc, argv, envp)
 	data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
 	debug = 0;
 #ifdef LOG_DAEMON
-	openlog("ftpd", LOG_PID, LOG_DAEMON);
+	openlog("ftpd", LOG_PID, FACILITY);
 #else
 	openlog("ftpd", LOG_PID);
 #endif
@@ -409,8 +422,7 @@ user(name)
 	 * elsewhere (that's why we set minfail to a huge number).
 	 */
 	if (last.failures > 0 && strcmp(name, last.name)) {
-		if (last.failures > last.minfail)
-			badlogin(last.name);
+		badlogin();
 		last.failures = 0;
 	}
 	strcpy(last.name, name);
@@ -442,7 +454,7 @@ user(name)
 		}
 		if ((fd = fopen(FTPUSERS, "r")) != NULL) {
 		    while (fgets(line, sizeof (line), fd) != NULL) {
-			if ((cp = index(line, '\n')) != NULL)
+			if ((cp = strchr(line, '\n')) != NULL)
 				*cp = '\0';
 			if (strcmp(line, name) == 0) {
 				reply(530, "User %s access denied.", name);
@@ -457,9 +469,9 @@ user(name)
 		(void) fclose(fd);
 	}
 	last.minfail = (pw ? 0 : 1);
-#ifdef SKEY
-	pwok = skeyaccess(name, (char *) 0, remotehost, addr_string);
-	reply(331, "%s", skey_challenge(name, pw, pwok));
+#ifdef KEY
+	pwok = keyaccess(pw, (char *) 0, remotehost, addr_string);
+	reply(331, "%s", key_challenge(name, pw, pwok));
 #else
 	reply(331, "Password required for %s.", name);
 #endif
@@ -502,8 +514,8 @@ pass(passwd)
 			salt = "xx";
 		else
 			salt = pw->pw_passwd;
-#ifdef SKEY
-		xpasswd = skey_crypt(passwd, salt, pw, pwok);
+#ifdef KEY
+		xpasswd = key_crypt(passwd, salt, pw, pwok);
 #else
 		xpasswd = crypt(passwd, salt);
 #endif
@@ -1067,6 +1079,7 @@ fatal(s)
 
 /* VARARGS2 */
 reply(n, fmt, p0, p1, p2, p3, p4, p5)
+	char *p0, *p1, *p2, *p3, *p4, *p5;
 	int n;
 	char *fmt;
 {
@@ -1082,6 +1095,7 @@ reply(n, fmt, p0, p1, p2, p3, p4, p5)
 
 /* VARARGS2 */
 lreply(n, fmt, p0, p1, p2, p3, p4, p5)
+	char *p0, *p1, *p2, *p3, *p4, *p5;
 	int n;
 	char *fmt;
 {
@@ -1113,7 +1127,7 @@ yyerror(s)
 {
 	char *cp;
 
-	if (cp = index(cbuf,'\n'))
+	if (cp = strchr(cbuf,'\n'))
 		*cp = '\0';
 	reply(500, "'%s': command not understood.", cbuf);
 }
@@ -1246,8 +1260,8 @@ dologout(status)
 		(void) seteuid((uid_t)0);
 		logwtmp(ttyline, "", "");
 	}
-	if (last.failures)
-		badlogin(last.name);
+	if (last.failures > 0)
+		badlogin();
 	/* beware of flushing buffers after a SIGPIPE */
 	_exit(status);
 }
@@ -1292,19 +1306,30 @@ passive()
 	int len;
 	register char *p, *a;
 
+	if (pdata >= 0)
+		close(pdata);
 	pdata = socket(AF_INET, SOCK_STREAM, 0);
 	if (pdata < 0) {
 		perror_reply(425, "Can't open passive connection");
 		return;
 	}
 	pasv_addr = ctrl_addr;
-	pasv_addr.sin_port = 0;
-	(void) seteuid((uid_t)0);
-	if (bind(pdata, (struct sockaddr *)&pasv_addr, sizeof(pasv_addr)) < 0) {
-		(void) seteuid((uid_t)pw->pw_uid);
-		goto pasv_error;
+#ifdef PASV_MIN_PORT
+	for (;;) {
+		static int pasv_port;
+		if (pasv_port < PASV_MIN_PORT || pasv_port > PASV_MAX_PORT)
+			pasv_port = PASV_MIN_PORT;
+		pasv_addr.sin_port = htons(pasv_port++);
+		if (bind(pdata, (struct sockaddr *)&pasv_addr, sizeof(pasv_addr)) >= 0)
+			break;
+		if (errno != EADDRINUSE)
+			goto pasv_error;
 	}
-	(void) seteuid((uid_t)pw->pw_uid);
+#else
+	pasv_addr.sin_port = 0;
+	if (bind(pdata, (struct sockaddr *)&pasv_addr, sizeof(pasv_addr)) < 0)
+		goto pasv_error;
+#endif
 	len = sizeof(pasv_addr);
 	if (getsockname(pdata, (struct sockaddr *) &pasv_addr, &len) < 0)
 		goto pasv_error;
@@ -1337,7 +1362,7 @@ gunique(local)
 {
 	static char new[MAXPATHLEN];
 	struct stat st;
-	char *cp = rindex(local, '/');
+	char *cp = strrchr(local, '/');
 	int count = 0;
 
 	if (cp)
@@ -1501,6 +1526,7 @@ send_file_list(whichfiles)
 
 /*VARARGS1*/
 setproctitle(fmt, a, b, c)
+char *a, *b, *c;
 char *fmt;
 {
 	register char *p, *bp, ch;
@@ -1549,10 +1575,9 @@ logxfer(name, size, start)
 
 
 
-badlogin(name)
-	char           *name;
+badlogin()
 {
-	if (last.failures && last.failures > last.minfail)
+	if (last.failures > 0 && last.failures > last.minfail)
 		syslog(LOG_ERR, "%d FTP LOGIN FAILURE%s from %s, %s",
 		       last.failures, last.failures > 1 ? "S" : "",
 		       remotehost, last.name);
