@@ -86,10 +86,13 @@ static char sccsid[] = "@(#)login.c	5.32.1.1 (Berkeley) 1/28/89";
 #include "sysv_shadow.h"
 #endif /* SYSV_SHADOW */
 #ifdef SYSV_LOGINDEFS
+#include <ulimit.h>
 #include "sysv_default.h"
 #endif /* SYSV_LOGINDEFS */
 
+#ifndef TTYGRPNAME
 #define	TTYGRPNAME	"tty"		/* name of group to own ttys */
+#endif
 
 #ifdef HAS_PATHS_H
 #include <paths.h>			/* Use system version if present */
@@ -99,6 +102,11 @@ static char sccsid[] = "@(#)login.c	5.32.1.1 (Berkeley) 1/28/89";
 #define	_PATH_MOTDFILE	"/etc/motd"
 #define	_PATH_HUSHLOGIN	".hushlogin"
 
+#ifdef ULTRIX_LAT
+#include <sys/ltatty.h>
+char	*checklat();
+#endif
+
 /* Ultrix syslog(3) has no facility stuff. */
 #ifndef LOG_AUTH
 #define LOG_AUTH	0
@@ -115,6 +123,21 @@ static char sccsid[] = "@(#)login.c	5.32.1.1 (Berkeley) 1/28/89";
 #define OXTABS	XTABS
 #endif
 
+/* SKEY or SNK interface definition: both use the same interface */
+#ifdef SKEY
+#define key_getpass skey_getpass
+#define key_crypt skey_crypt
+#define keyaccess skeyaccess
+#define KEY
+#endif /* SKEY */
+
+#ifdef SNK
+#define key_getpass snk_getpass
+#define key_crypt snk_crypt
+#define keyaccess snkaccess
+#define KEY
+#endif /* SNK */
+
 static	char rusername[100], lusername[100];
 
 /*
@@ -138,6 +161,8 @@ struct	termios termios;
 #define ECHOPRT	0
 #define ECHOCTL	0
 #define ECHOKE	0
+#undef	ECHOK		/* MHC prevent poor response when doing ^U */
+#define ECHOK	0
 #define IMAXBEL	0
 #endif
 
@@ -174,10 +199,10 @@ main(argc, argv)
 	struct rlimit old_core_limit;
 	struct rlimit new_core_limit;
 #endif
-#ifdef SKEY
+#ifdef KEY
 	int permit_passwd = 0;
-	char *skey_getpass(), *skey_crypt();
-#endif /* SKEY */
+	char *key_getpass(), *key_crypt();
+#endif /* KEY */
 	extern int _check_rhosts_file;
 #ifdef SYSV_LOGINDEFS
 	int mask;
@@ -227,6 +252,10 @@ main(argc, argv)
 	fflag = hflag = pflag = 0;
 	rflag = 0;
 	passwd_req = 1;
+#ifdef ULTRIX_LAT
+	/* Get server/port case of LAT login */
+	hostname = checklat();
+#endif
 	while ((ch = getopt(argc, argv, "d:fh:lpr:R:")) != EOF)
 		switch (ch) {
 #ifdef SYSV4 /* Allow but always ignore the -d option. */
@@ -340,7 +369,12 @@ main(argc, argv)
 	termios.c_lflag &= ~(ECHOPRT|TOSTOP|FLUSHO);
 	termios.c_oflag |= (OPOST|ONLCR);
 	termios.c_oflag &= ~OXTABS;
-	termios.c_cc[VEOF] = 4;
+#define Ctl(x) ((x) ^ 0100)
+	termios.c_cc[VEOF] = Ctl('D');
+	if (termios.c_cc[VERASE] == '#')
+		termios.c_cc[VERASE] = Ctl('H');
+	if (termios.c_cc[VKILL] == '@')
+		termios.c_cc[VKILL] = Ctl('U');
 	(void)tcsetattr(0, TCSANOW, &termios);
 
 	/*
@@ -402,14 +436,14 @@ main(argc, argv)
 			break;
 
 		setpriority(PRIO_PROCESS, 0, -4);
-#ifdef SKEY
-		permit_passwd = skeyaccess(username, tty, hostname, (char *) 0);
-		pp = skey_getpass("Password:", pwd, permit_passwd);
-		p = skey_crypt(pp, salt, pwd, permit_passwd);
-#else /* SKEY */
+#ifdef KEY
+		permit_passwd = keyaccess(pwd, tty, hostname, (char *) 0);
+		pp = key_getpass("Password:", pwd, permit_passwd);
+		p = key_crypt(pp, salt, pwd, permit_passwd);
+#else /* KEY */
 		pp = getpass("Password:");
 		p = crypt(pp, salt);
-#endif /* SKEY */
+#endif /* KEY */
 		setpriority(PRIO_PROCESS, 0, 0);
 
 #if !defined(DES_RPC) && !defined(KERBEROS) /* Need password later. */
@@ -522,11 +556,6 @@ main(argc, argv)
 		(void)ioctl(0, TIOCSWINSZ, &win);
 	}
 
-#ifdef SYSV_LOGINDEFS
-	if (default_ulimit && ulimit(2, atol(default_ulimit)) < 0)
-		printf("Could not set ULIMIT to %ld\n", default_ulimit);
-#endif /* SYSV_LOGINDEFS */
-
 	/*
 	 * Set device protections, depending on what terminal the
 	 * user is logged in. This feature is used on Suns to give
@@ -564,8 +593,10 @@ main(argc, argv)
 #endif /* KERBEROS */
 
 #ifdef DES_RPC /* Do moral equivalent of keylogin. */
-	if (pp != 0)
+	if (pp != 0) {
+		pp[PASSWD_LENGTH] = 0;
 		login_desrpc(pp);
+	}
 #endif /* DES_RPC */
 
 #if defined(DES_RPC) || defined(KERBEROS)
@@ -1000,43 +1031,6 @@ sleepexit(eval)
 	exit(eval);
 }
 
-#ifdef	NO_SETENV
-
-setenv(name, value, overwrite)
-char   *name;
-char   *value;
-int     overwrite;
-{
-	char   *p;
-
-	if (overwrite == 0 && getenv(name) != 0)
-		return (0);
-	if ((p = malloc(strlen(name) + strlen(value) + 2)) == 0) {
-		fprintf(stderr, "out of memory\n");
-		sleepexit(1);
-	}
-	sprintf(p, "%s=%s", name, value);
-	return (putenv(p));
-}
-
-#endif /* NO_SETENV */
-
-fixenv(cpp)
-char  **cpp;
-{
-	register char **xpp;
-	register char *cp;
-
-	while (cp = *cpp) {
-		if (strncmp(cp, "LD_", 3) == 0 || strncmp(cp, "IFS=", 4) == 0) {
-			for (xpp = cpp; xpp[0] = xpp[1]; xpp++);
-			 /* void */ ;
-		} else {
-			cpp++;
-		}
-	}
-}
-
 #if defined(SYSV_SHADOW) || defined(SYSV_LOGINDEFS)
 
 change_passwd(who)
@@ -1061,3 +1055,25 @@ change_passwd(who)
 }
 
 #endif /* SYSV_SHADOW || SYSV_LOGINDEFS */
+
+#ifdef ULTRIX_LAT
+
+char *
+checklat()
+{
+	struct ltattyi  ltainfo;
+	static char     lat_hostport[MAXLTASERVSIZE + MAXLTAPORTSIZE + 2];
+
+	lat_hostport[0] = '\0';
+
+	if (ioctl(0, LIOCTTYI, &ltainfo) >= 0) {
+		strcpy(lat_hostport, ltainfo.lta_server_name);
+		strcat(lat_hostport, "/");
+		strcat(lat_hostport, ltainfo.lta_server_port);
+		return (lat_hostport);
+	} else {
+		return (0);
+	}
+}
+
+#endif /* ULTRIX_LAT */
