@@ -79,14 +79,14 @@ extern	char	*ptsname();
 #define PTY_PKT_READ	read
 #endif
 
-#ifndef FIONBIO		/* XXX */
-#include <sys/filio.h>
-#endif
-
 #ifndef TIOCSWINSZ	/* XXX */
 #include <sys/ioctl.h>
 #endif
 
+#ifndef FIONBIO		/* XXX */
+#include <sys/filio.h>
+#endif
+
 #ifndef NEW_LOGIN
 #define OLD_LOGIN	/* use the -r option of /bin/login */
 #endif
@@ -129,6 +129,11 @@ struct	passwd *getpwnam(), *pwd;
 
 char    *procname;
 
+/* On some systems, open ptys become wedged when the process exits. */
+#ifdef CLOSE_PTY_MASTER
+int	pty;
+#endif /* CLOSE_PTY_MASTER */
+
 main(argc, argv)
 	int argc;
 	char **argv;
@@ -188,7 +193,6 @@ doit(f, fromp)
 	int authenticated = 0, hostok = 0;
 	char remotehost[2 * MAXHOSTNAMELEN + 1];
 	register struct hostent *hp;
-	struct hostent hostent;
 	char c;
 	struct termios termios;
 
@@ -202,11 +206,7 @@ doit(f, fromp)
 	hp = gethostbyaddr((char *) &fromp->sin_addr, sizeof (struct in_addr),
 		fromp->sin_family);
 	if (hp == 0) {
-		/*
-		 * Only the name is used below.
-		 */
-		hp = &hostent;
-		hp->h_name = inet_ntoa(fromp->sin_addr);
+		strcpy(remotehost, inet_ntoa(fromp->sin_addr));
 		hostok++;
 	} else {
 		/*
@@ -217,22 +217,20 @@ doit(f, fromp)
 		strncpy(remotehost, hp->h_name, sizeof(remotehost) - 1);
 		remotehost[sizeof(remotehost) - 1] = 0;
 		hp = gethostbyname(remotehost);
-		if (hp)
+		if (hp) {
 		    for (; hp->h_addr_list[0]; hp->h_addr_list++)
 			if (!bcmp(hp->h_addr_list[0], (caddr_t)&fromp->sin_addr,
 			    sizeof(fromp->sin_addr))) {
 				hostok++;
 				break;
 			}
-			if (hostok == 0)
-				syslog(LOG_NOTICE, 
-					"Host addr %s not listed for host %s", 
-					inet_ntoa(fromp->sin_addr), remotehost);
-		else {
-		     hp = &hostent;
 		}
-		/* If host name verification fails use the address instead. */
-		hp->h_name = hostok ? remotehost : inet_ntoa(fromp->sin_addr);
+		if (hostok == 0) {
+			syslog(LOG_NOTICE, 
+				"Host addr %s not listed for host %s", 
+				inet_ntoa(fromp->sin_addr), remotehost);
+			strcpy(remotehost, inet_ntoa(fromp->sin_addr));
+		}
 	}
 
 	if (fromp->sin_family != AF_INET ||
@@ -269,14 +267,14 @@ doit(f, fromp)
       }
 #endif
 	write(f, "", 1);
-	if (do_rlogin(hp->h_name) == 0) {
+	if (do_rlogin(remotehost) == 0) {
 		if (hostok)
 		    authenticated++;
 		else
 		    write(f, "rlogind: Host address mismatch.\r\n",
 		     sizeof("rlogind: Host address mismatch.\r\n") - 1);
 	}
-	do_access(procname, hp->h_name, inet_ntoa(fromp->sin_addr), 
+	do_access(procname, remotehost, inet_ntoa(fromp->sin_addr), 
 		  rusername, lusername);
 
 #ifdef STREAM_PTY
@@ -362,21 +360,21 @@ gotpty:
 #endif /* SYSV_UTMP */
 #ifdef OLD_LOGIN
 		execl(_PATH_LOGIN, "login", _check_rhosts_file ? "-r" : "-lr",
-			hp->h_name, (char *) 0);
+			remotehost, (char *) 0);
 #else /* OLD_LOGIN */
 #ifndef SYSV_ENV
 		if (authenticated)
-			execl(_PATH_LOGIN, "login", "-p", "-h", hp->h_name,
+			execl(_PATH_LOGIN, "login", "-p", "-h", remotehost,
 			    "-f", lusername, (char *) 0);
 		else
-			execl(_PATH_LOGIN, "login", "-p", "-h", hp->h_name,
+			execl(_PATH_LOGIN, "login", "-p", "-h", remotehost,
 			    lusername, (char *) 0);
 #else /* SYSV_ENV Does not need -p to preserve TERM */
 		if (authenticated)
-			execl(_PATH_LOGIN, "login", "-h", hp->h_name,
+			execl(_PATH_LOGIN, "login", "-h", remotehost,
 			    "-f", lusername, (char *) 0);
 		else
-			execl(_PATH_LOGIN, "login", "-h", hp->h_name,
+			execl(_PATH_LOGIN, "login", "-h", remotehost,
 			    lusername, (char *) 0);
 #endif /* SYSV_ENV */
 #endif /* OLD_LOGIN */
@@ -393,6 +391,9 @@ gotpty:
 	signal(SIGTSTP, SIG_IGN);
 	signal(SIGCHLD, cleanup);
 	setsid();
+#ifdef CLOSE_PTY_MASTER
+	pty = p;
+#endif /* CLOSE_PTY_MASTER */
 	protocol(f, p);
 	signal(SIGCHLD, SIG_IGN);
 	cleanup();
@@ -580,6 +581,9 @@ void	cleanup()
 	(void)chmod(line, 0666);
 	(void)chown(line, 0, 0);
 #endif /* SYSV_UTMP */
+#ifdef CLOSE_PTY_MASTER
+	close(pty);
+#endif /* CLOSE_PTY_MASTER */
 	shutdown(netf, 2);
 	exit(1);
 }
