@@ -72,6 +72,10 @@ extern	char	*ptsname();
 #include <sys/ioctl.h>
 #endif
 
+#ifndef TIOCSWINSZ      /* XXX */
+#include <sys/ioctl.h>
+#endif
+
 #if !defined(STREAM_PTY) && !defined(TIOCPKT)
 #include <sys/pty.h>
 #endif
@@ -81,9 +85,7 @@ extern	char	*ptsname();
 #define rindex		strrchr
 #define bcopy(s,d,l)	memcpy(d,s,l)
 
-#ifdef __STDC__
 #define puts		xputs		/* prototype conflict */
-#endif
 
 /* Ultrix syslog(3) has no facility stuff. */
 #ifndef LOG_DAEMON
@@ -106,6 +108,10 @@ extern	char	*ptsname();
 char	hisopts[256];
 char	myopts[256];
 
+#ifndef TELOPT_NAWS
+#define TELOPT_NAWS		31		/* window size */
+#endif
+
 char	doopt[] = { IAC, DO, '%', 'c', 0 };
 char	dont[] = { IAC, DONT, '%', 'c', 0 };
 char	will[] = { IAC, WILL, '%', 'c', 0 };
@@ -140,10 +146,14 @@ int	not42 = 1;
 #define	BANNER  "\r\n\r\nULTRIX V4.2A (Rev. 47) (%s)\r\n\r\r\n\r"
 #endif
 
-#ifdef HPUX	/* uname -s hostname uname -r uname -v uname -m (tty) */
+#ifdef HPUX9	/* uname -s hostname uname -r uname -v uname -m (tty) */
 #define	BANNER  "\r\n\r\nHP-UX %s\r\n\r\r\n\r"
 #endif
 
+#ifdef OSF1
+#define BANNER	"\r\n\r\nOSF/1 %s\r\n\r\r\n\r"
+#endif
+
 #ifndef BANNER
 #define	BANNER	"\r\n\r\n4.3 BSD UNIX (%s)\r\n\r\r\n\r"
 #endif
@@ -238,7 +248,7 @@ main(argc, argv)
 	    close(s);
 	}
 #endif	/* defined(DEBUG) */
-	openlog("telnetd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
+	openlog("telnetd", LOG_PID | LOG_ODELAY, FACILITY);
 	fromlen = sizeof (from);
 	if (getpeername(0, (struct sockaddr *) &from, &fromlen) < 0) {
 		fprintf(stderr, "%s: ", argv[0]);
@@ -322,7 +332,7 @@ doit(f, who)
 	int f;
 	struct sockaddr_in *who;
 {
-	char *host, *inet_ntoa();
+	char host[BUFSIZ], *inet_ntoa();
 	int i, p, t;
 #ifdef ultrix
 	struct termios b;
@@ -378,9 +388,9 @@ gotpty:
 	hp = gethostbyaddr((char *) &who->sin_addr, sizeof (struct in_addr),
 		who->sin_family);
 	if (hp)
-		host = hp->h_name;
+		strncpy(host, hp->h_name, sizeof(host) - 1);
 	else
-		host = inet_ntoa(who->sin_addr);
+		strcpy(host, inet_ntoa(who->sin_addr));
 
 	net = f;
 	pty = p;
@@ -891,6 +901,7 @@ telrcv()
 			 * Begin option subnegotiation...
 			 */
 			case SB:
+				SB_CLEAR();	/* this one was missing */
 				state = TS_SB;
 				continue;
 
@@ -1010,6 +1021,10 @@ willoption(option)
 		fmt = doopt;
 		break;
 
+	case TELOPT_NAWS:
+		fmt = doopt;
+		break;
+
 	case TELOPT_TM:
 		fmt = dont;
 		break;
@@ -1077,6 +1092,10 @@ dooption(option)
 		fmt = will;
 		break;
 
+	case TELOPT_NAWS:
+		fmt = will;
+		break;
+
 	default:
 		fmt = wont;
 		break;
@@ -1129,6 +1148,7 @@ int option;
 
 suboption()
 {
+
     switch (SB_GET()) {
     case TELOPT_TTYPE: {		/* Yaaaay! */
 	static char terminalname[5+41] = "TERM=";
@@ -1156,6 +1176,30 @@ suboption()
 	break;
     }
 
+    case TELOPT_NAWS: {
+	struct winsize ws;
+#ifdef STREAM_PTY
+	int pty = pts;	/* XXX apply ioctl()s at slave end */
+#endif
+
+	if (hisopts[TELOPT_NAWS] == OPT_NO)	/* ignore disabled option */
+	    break;
+	if (SB_EOF())
+	    return;
+	ws.ws_col = SB_GET() << 8;
+	if (SB_EOF())
+	    return;
+	ws.ws_col |= SB_GET();
+	if (SB_EOF())
+	    return;
+	ws.ws_row = SB_GET() << 8;
+	if (SB_EOF())
+	    return;
+	ws.ws_row |= SB_GET();
+	(void) ioctl(pty, TIOCSWINSZ, (char *) &ws);
+	break;
+    }
+
     default:
 	;
     }
@@ -1442,6 +1486,9 @@ void	cleanup()
 	(void)chmod(line, 0666);
 	(void)chown(line, 0, 0);
 #endif /* SYSV_UTMP */
+#ifdef CLOSE_PTY_MASTER
+	close(pty);
+#endif /* CLOSE_PTY_MASTER */
 	shutdown(net, 2);
 	exit(1);
 }
